import 'colors';
import spec from './spec.type';
import test from './test.type';
import Action from './action.i';
import Assert from './assert.i';
import Equals from './asserts/equals';
import GreaterThan from './asserts/greater-than';
import LessThan from './asserts/less-than';
import Includes from './asserts/includes';
export { spec, test, Action, Assert, Equals, GreaterThan, LessThan, Includes };
/**
 * Finds all files in the given path that match the given extension.
 * Looks through directories and sub directories. Default extension is '.test.js'.
 * @param path the path to search in
 * @param extensions a string array of extensions to look for
 * @returns a promise with an array of paths to found files
 */
export declare function findFiles(filePath: string, extensions?: string[]): Promise<string[]>;
/**
 * Stores the given value under the given key.
 * @param key the key to identify the store item with
 * @param func the function to call that returns value to store
 */
export declare function load(key: string, func: Function): void;
/**
 * Stores the given test.
 * @param title the name of the test
 * @param specs an array of test specs
 */
export declare function addTest(title: string, specs: spec[]): void;
/**
 * Registers and stores actions.
 * @param action an action to register
 */
export declare function registerAction(action: any): void;
/**
 * Registers and stores an array of actions.
 * @param action an action to register
 */
export declare function registerActions(...actions: any[]): void;
/**
 * Registers and stores asserts.
 * @param assert an assert to register
 */
export declare function registerAssert(assert: any): void;
/**
 * Registers and stores an array of asserts.
 * @param assert an assert to register
 */
export declare function registerAsserts(...asserts: any[]): void;
/**
 * Executes an array of specs using the current context.
 * @param specs an array of specs.
 */
export declare function runSpecs(specs: spec[]): Promise<void>;
/**
 * Runs the currently loaded set of tests.
 */
export declare function runTests(): Promise<void>;
