"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("colors");
var globby = require("globby");
var equals_1 = require("./asserts/equals");
exports.Equals = equals_1.default;
var greater_than_1 = require("./asserts/greater-than");
exports.GreaterThan = greater_than_1.default;
var less_than_1 = require("./asserts/less-than");
exports.LessThan = less_than_1.default;
var includes_1 = require("./asserts/includes");
exports.Includes = includes_1.default;
var fs = require('fs');
var path = require('path');
var STORE = {
    context: {},
    reporters: [],
    variables: {},
};
var ACTIONS = [];
var ASSERTS = [];
var TESTS = [];
/**
 * Finds all files in the given path that match the given extension.
 * Looks through directories and sub directories. Default extension is '.test.js'.
 * @param path the path to search in
 * @param extensions a string array of extensions to look for
 * @returns a promise with an array of paths to found files
 */
function findFiles(filePath, extensions) {
    if (extensions === void 0) { extensions = ['.test.js']; }
    return __awaiter(this, void 0, void 0, function () {
        var fileTypes, paths;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!path.isAbsolute(filePath)) {
                        filePath = process.cwd() + "/" + filePath;
                    }
                    if (!isFile(filePath)) return [3 /*break*/, 1];
                    return [2 /*return*/, [filePath]];
                case 1:
                    if (!isDirectory(filePath)) return [3 /*break*/, 3];
                    if (!filePath.endsWith('/')) {
                        filePath += '/';
                    }
                    fileTypes = extensions.map(function (ext) { return '**/*' + ext; });
                    return [4 /*yield*/, globby(fileTypes, {
                            cwd: filePath,
                        })];
                case 2:
                    paths = _a.sent();
                    paths = paths.map(function (path) { return filePath + path; });
                    return [2 /*return*/, paths];
                case 3:
                    console.error('Error: Invalid path specified'.red);
                    return [2 /*return*/, []];
            }
        });
    });
}
exports.findFiles = findFiles;
/**
 * Checks if a given path is a file
 * @param path the path to check
 */
function isFile(path) {
    return fs.existsSync(path) && fs.statSync(path).isFile();
}
/**
 * Checks if a given path is a directory
 * @param path the path to check
 */
function isDirectory(path) {
    return fs.existsSync(path) && fs.statSync(path).isDirectory();
}
/**
 * Stores the given value under the given key.
 * @param key the key to identify the store item with
 * @param func the function to call that returns value to store
 */
function load(key, func) {
    var val = func.call(null);
    switch (key) {
        case 'context':
            STORE.context = __assign(__assign({}, STORE.context), val);
            return;
        case 'reporter':
            STORE.reporters.push(val);
            return;
        default:
            return;
    }
}
exports.load = load;
/**
 * Stores the given test.
 * @param title the name of the test
 * @param specs an array of test specs
 */
function addTest(title, specs) {
    TESTS.push({ title: title, specs: specs });
}
exports.addTest = addTest;
/**
 * Gets the value stored under the context key.
 */
function getContext() {
    return STORE.context;
}
/**
 * Gets the value stored under the reporter key.
 */
// TODO: The following ts-ignore comment has been added to temporarily suppress compilation errors
// as the function is not currently used. Remove this when this function has been used somewhere.
// @ts-ignore
function getReporters() {
    return STORE.reporters;
}
/**
 * Registers and stores actions.
 * @param action an action to register
 */
function registerAction(action) {
    if (!ACTIONS.includes(action) && isAction(action)) {
        ACTIONS.push(action);
    }
}
exports.registerAction = registerAction;
/**
 * Typeguard to check if an instance of any class that implements
 * the Action interface also implements it's methods.
 * @param object the object to check
 */
function isAction(obj) {
    return typeof obj.prototype.execute === 'function';
}
/**
 * Registers and stores an array of actions.
 * @param action an action to register
 */
function registerActions() {
    var actions = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        actions[_i] = arguments[_i];
    }
    actions.forEach(function (action) {
        registerAction(action);
    });
}
exports.registerActions = registerActions;
/**
 * Registers and stores asserts.
 * @param assert an assert to register
 */
function registerAssert(assert) {
    if (!ASSERTS.includes(assert) && isAssert(assert)) {
        ASSERTS.push(assert);
    }
}
exports.registerAssert = registerAssert;
/**
 * Typeguard to check if any class that implements the Action interface
 * also implements it's methods.
 * @param object the object to check
 */
function isAssert(obj) {
    return typeof obj.prototype.execute === 'function';
}
/**
 * Registers and stores an array of asserts.
 * @param assert an assert to register
 */
function registerAsserts() {
    var asserts = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        asserts[_i] = arguments[_i];
    }
    asserts.forEach(function (assert) {
        registerAssert(assert);
    });
}
exports.registerAsserts = registerAsserts;
/**
 * Executes an array of specs using the current context.
 * @param specs an array of specs.
 */
function runSpecs(specs) {
    return __awaiter(this, void 0, void 0, function () {
        var _i, specs_1, spec, context, action, args, outs, results, error_1, assert, args, result, error_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _i = 0, specs_1 = specs;
                    _a.label = 1;
                case 1:
                    if (!(_i < specs_1.length)) return [3 /*break*/, 13];
                    spec = specs_1[_i];
                    if (!(spec.action && ACTIONS.includes(spec.action))) return [3 /*break*/, 6];
                    context = getContext();
                    action = new spec.action();
                    args = getVariables(spec.args);
                    outs = spec.outs;
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, action.execute(args, context)];
                case 3:
                    results = _a.sent();
                    if (outs && results) {
                        storeVariables(outs, results);
                    }
                    console.log(("\u2713 SUCCESS Action: " + spec.title).green);
                    return [3 /*break*/, 5];
                case 4:
                    error_1 = _a.sent();
                    console.error(("\u2717 ERROR in Action: " + spec.title).red);
                    return [3 /*break*/, 5];
                case 5: return [3 /*break*/, 12];
                case 6:
                    if (!(spec.assert && ASSERTS.includes(spec.assert))) return [3 /*break*/, 11];
                    assert = new spec.assert();
                    args = getVariables(spec.args);
                    _a.label = 7;
                case 7:
                    _a.trys.push([7, 9, , 10]);
                    return [4 /*yield*/, assert.execute(args)];
                case 8:
                    result = _a.sent();
                    if (result) {
                        console.log(("\u2713 SUCCESS Assert: " + spec.title + ", values: " + args).green);
                    }
                    else {
                        console.error(("\u2717 FAILED Assert: " + spec.title + ", values: " + args).red);
                    }
                    return [3 /*break*/, 10];
                case 9:
                    error_2 = _a.sent();
                    console.error(("ERROR in Assert: " + spec.title).red);
                    return [3 /*break*/, 10];
                case 10: return [3 /*break*/, 12];
                case 11: return [3 /*break*/, 12];
                case 12:
                    _i++;
                    return [3 /*break*/, 1];
                case 13: return [2 /*return*/];
            }
        });
    });
}
exports.runSpecs = runSpecs;
/**
 * Gets stored tests.
 */
// TODO: The following ts-ignore comment has been added to temporarily suppress compilation errors
// as the function is not currently used. Remove this when this function has been used somewhere.
// @ts-ignore
function getTests() {
    return TESTS;
}
/**
 * Gets stored variables if they exist, else returns the key value,
 * for a set of given keys.
 * @param keys the keys to retrieve variable values for.
 */
function getVariables(keys) {
    if (!keys || !keys.length) {
        return [];
    }
    return keys.map(function (key) {
        if (typeof key === 'string' && key.startsWith('$') && STORE.variables[key] !== undefined) {
            return STORE.variables[key];
        }
        else {
            return key;
        }
    });
}
/**
 * Stores a set of results under the given keys.
 * @param keys the keys to be used to retrieve stored values.
 * @param results the values to be stored
 */
function storeVariables(keys, results) {
    if (!keys || !results || keys.length > results.length) {
        console.log('Error: Mismtach in number of outs and keys'.red);
    }
    keys.forEach(function (key, index) {
        if (key.startsWith('$')) {
            STORE.variables[key] = results[index];
        }
    });
}
/**
 * Runs the currently loaded set of tests.
 */
function runTests() {
    return __awaiter(this, void 0, void 0, function () {
        var _i, TESTS_1, test_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _i = 0, TESTS_1 = TESTS;
                    _a.label = 1;
                case 1:
                    if (!(_i < TESTS_1.length)) return [3 /*break*/, 4];
                    test_1 = TESTS_1[_i];
                    console.log("Running test " + test_1.title);
                    return [4 /*yield*/, runSpecs(test_1.specs).catch(function (err) { return console.log('Error: ', err); })];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.runTests = runTests;
//# sourceMappingURL=main.js.map